import arxiv
from datetime import datetime, timedelta
import json
import os
import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import multiprocessing
from .download import down_load

def create_session_with_proxy(proxy):
    session = requests.Session()
    session.proxies.update(proxy)
    
    retries = Retry(total=5, backoff_factor=0.1, status_forcelist=[502, 503, 504])
    session.mount('http://', HTTPAdapter(max_retries=retries))
    session.mount('https://', HTTPAdapter(max_retries=retries))
    
    return session

def multi_processing(results, categories, keywords, proxy):
    processes = []
    for category in categories:
        p = multiprocessing.Process(target=down_load, args=(results, keywords, proxy))
        processes.append(p)
        p.start()

    for p in processes:
        p.join()


def fetch_arxiv_updates(categories, keywords, proxy, download_mode, days):
    search_query = ' OR '.join([f'cat:{cat}' for cat in categories])
    if not proxy == "":
        proxies = {
            'http': f'{proxy}',
            'https': f'{proxy}',
        }
        print("using proxy")
        session_with_proxy = create_session_with_proxy(proxies)
        arxiv.Client._session = session_with_proxy

    search = arxiv.Search(search_query,max_results = 1000,sort_by = arxiv.SortCriterion.SubmittedDate)
    print("searching finished")
    results = []
    result = []
    cnt = 0
    for passage in search.results():
        cnt += 1
        if passage.updated.date()>datetime.now().date()-timedelta(days=days):
            if any(keyword.lower() in passage.summary.lower() for keyword in keywords):
                results.append({
                    "title": passage.title,
                    "authors": [str(author) for author in passage.authors],
                    'published': passage.published,
                    'link': passage.entry_id,
                    'summary': passage.summary,
                    'category': categories,
                    'keyword': [keyword for keyword in keywords if keyword.lower() in passage.summary.lower()]
                })
                result.append({
                    "title": passage.title,
                    "authors": [str(author) for author in passage.authors],
                    'published': passage.published,
                    'link': passage.entry_id,
                    'summary': passage.summary,
                    'category': categories,
                    'keyword': [keyword for keyword in keywords if keyword.lower() in passage.summary.lower()]
                })
        if(cnt%100 == 0):
            down_load(result, keywords, proxy, download_mode)
            result = []
    if not result == []:
        down_load(result, keywords, proxy, download_mode)
    print("process finished")
    return  results

def load_papers_from_db():
    import sqlite3
    cwd = os.getcwd()
    with open(os.path.join(cwd, "local.txt"), 'r') as file:
        local=file.read()
    db_path = os.path.join(local, "download.db")

    if not os.path.exists(db_path):
        print(f"Database file does not exist at: {db_path}")
    else:
        results=[]
        try:
            
            conn = sqlite3.connect(db_path)
            cursor = conn.cursor()
            query = f"SELECT * FROM papers "

            cursor.execute(query)
            rows = cursor.fetchall()

            
            for row in rows:
                results.append({
                    "title": row[1],
                    'published': row[4],
                    'link': row[0],
                    'keyword': json.loads(row[2]),
                    'local_link': row[3]
                })
        except sqlite3.Error as e:
            print(f"An error occurred: {e}")
        finally:
            if conn:
                conn.close()   
    return results
        
    
        
    



import argparse
import subprocess
import multiprocessing
import schedule
import time
import os
from datetime import datetime
from .src.fetch import fetch_arxiv_updates
from .src.send_email import send_email
from .src.download import is_paper_in_db
from .src.download import init_db


def fetch_job(categories, keywords, proxy, email_sender, email_password, email_receiver, smtp_server, smtp_port, download_mode, days):
    results = fetch_arxiv_updates(categories, keywords, proxy, download_mode, days)
   
    init_db()
    new_papers = [paper for paper in results if not is_paper_in_db(paper['link'])]

    if email_sender == "" or email_password == "" or email_receiver == "" or smtp_server == "" or smtp_port == "":
        print("incomplete info for sending email")
    else:
        send_email(new_papers, email_sender, email_password, email_receiver, smtp_server, smtp_port)
    
    
    print("updated")
    print(str(datetime.now().date()))

def build_web(download_mode, keywords):
    time.sleep(1)
    subprocess.run(['python', 'fetch/webpage.py', download_mode, str(len(keywords))]+ keywords)

def main():
    parser = argparse.ArgumentParser(description='Process category and keywords.')
    parser.add_argument('--category', nargs='+', default=['cs.CV', 'cs.RO'], help='List of categories')
    parser.add_argument('--keywords', nargs='+', default=['radiance field',"deep learning"], help='List of keywords')
    parser.add_argument('--proxy', type=str, default="", help='Proxy settings')
    parser.add_argument('--email_sender', type=str, default="", help='Sender email address')
    parser.add_argument('--email_password', type=str, default="", help='Sender email password')
    parser.add_argument('--email_receiver', type=str, default="", help='Receiver email address')
    parser.add_argument('--frequency', type=str, default="", help='regular update')
    parser.add_argument('--smtp_server', type=str, default="", help="your smtp server's address")
    parser.add_argument('--smtp_port', type=str, default="", help='smtp port')
    parser.add_argument('--download_mode', type=str, default="1", help='0=download all when fetching, 1=download when clicking web link,  default=1')
    parser.add_argument('--days', type=int, default=3, help='number of days you want to trace, recommended <= 7, default=3')
    parser.add_argument('--view_keywords', nargs='+', default=[], help='List of keywords you want to view at the website')
    parser.add_argument('--local', type = str, default=".local/arxivfans", help="where you want to save the papers")
    args = parser.parse_args()
    categories = args.category
    keywords = args.keywords
    proxy = args.proxy
    email_sender = args.email_sender
    email_password = args.email_password
    email_receiver = args.email_receiver
    smtp_server = args.smtp_server
    smtp_port = args.smtp_port
    frequency = args.frequency
    download_mode = args.download_mode
    days = args.days
    local = args.local
    with open(os.path.join(os.getcwd(), "local.txt"), 'w') as file:
        file.write(local)

    if args.view_keywords == []:
        view_keywords = keywords
    else:
        view_keywords = args.view_keywords
    print(f"you are searching {categories} for {keywords}.")
    p = multiprocessing.Process(target = build_web, args=(str(download_mode), view_keywords))
    p.start()
    print(f"updating...")
    if frequency.lower() == "daily":
        schedule.every().day.at("08:00").do(fetch_job, categories, keywords, proxy, email_sender, email_password, email_receiver, smtp_server, smtp_port, download_mode, days)
        print("scheduled at 8 a.m. everyday\n")
        while True:
            schedule.run_pending()
            
            time.sleep(60)
    else:
        
        
        fetch_job(categories, keywords, proxy, email_sender, email_password, email_receiver, smtp_server, smtp_port, download_mode, days)
    p.join()
        
    

if __name__=="__main__":
    main()



from flask import Flask, render_template_string, jsonify, request, send_from_directory
from .src.fetch import load_papers_from_db
import os
from .src.download import save_paper
import webbrowser
import sys
import threading
global download_mode
global keywords_show

app = Flask(__name__)

def get_updates():
    updates = load_papers_from_db()
    if not updates:
        return []
    updates.sort(key=lambda x: x['published'], reverse=True)
    return updates

def handle_link_click(url):
    if download_mode == "1" or download_mode == "2":
        save_paper(url)
    return {"status": "success", "message": f"Handled link click for {url}"}

def handle_download(url):
    filename = save_paper(url)
    return {"status": "success", "message": f"Paper downloaded from {url}", "local_link": filename}

@app.route('/')
def index():
    updates = get_updates()
    template = '''
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Paper List</title>
    <style>
        body {
            font-size: 20px;
            font-family: Arial, sans-serif;
            background-color: #f9f9f9;
            margin: 0;
            padding: 20px;
        }
        h1 {
            text-align: center;
        }
        .container {
            max-width: 1800px;
            margin: 0 auto;
            background: #fff;
            padding: 20px;
            box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
        }
        .checkbox-group {
            max-height: 150px;
            overflow-y: auto;
            border: 1px solid #ddd;
            padding: 10px;
            margin-bottom: 20px;
        }
        .checkbox-item {
            display: flex;
            align-items: center;
            margin: 5px 0;
        }
        .checkbox-item input {
            margin-right: 10px;
        }
        .checkbox-label {
            font-size: 1.1em;
        }
        table {
            width: 100%;
            border-collapse: collapse;
        }
        table, th, td {
            border: 1px solid #ddd;
        }
        th, td {
            padding: 10px;
            text-align: left;
        }
        th {
            background-color: #f2f2f2;
        }
        tr:hover {
            background-color: #f1f1f1;
        }
        .refresh-button {
            display: block;
            margin: 20px auto;
            padding: 10px 20px;
            background-color: #4CAF50;
            color: white;
            border: none;
            border-radius: 4px;
            cursor: pointer;
            font-size: 1em;
        }
        .refresh-button:hover {
            background-color: #45a049;
        }
        .download-button {
            display: inline-block;
            padding: 5px 10px;
            background-color: #2196F3;
            color: white;
            border: none;
            border-radius: 4px;
            cursor: pointer;
            font-size: 0.9em;
        }
        .download-button:hover {
            background-color: #1E88E5;
        }
        .local-link-cell {
            width: 150px; 
        }
    </style>
</head>
<body>
    <div class="container">
        <h1>Paper List</h1>
        <button class="refresh-button" onclick="fetchUpdates()">Refresh</button>
        <div class="checkbox-group" id="checkbox-group">
            <!-- Dynamic checkbox items will be added here -->
        </div>
        <table>
            <thead>
                <tr>
                    <th>Title</th>
                    <th>Published date</th>
                    <th>Keywords</th>
                    <th>Link</th>
                    <th class="local-link-cell">Local link</th>
                </tr>
            </thead>
            <tbody id="paper-table-body">
                <!-- Paper rows will be added here -->
            </tbody>
        </table>
    </div>

    <script>
        let currentSelectedKeywords = [];
        const keywordsShow = {{ keywords_show|tojson }};

        function fetchUpdates() {
            fetch('/get-updates')
                .then(response => response.json())
                .then(updates => {
                    const results = updates.map(paper => ({
                        title: paper['title'],
                        published: paper['published'],
                        keywords: paper['keyword'],
                        link: paper['link'],
                        local_link: paper['local_link']
                    }));

                    console.log("Results:", results);  // Debug line to print results

                    // Extract unique keywords and filter by keywordsShow
                    const keywords = [...new Set(results.flatMap(paper => paper.keywords))]
                        .filter(keyword => keywordsShow.includes(keyword));

                    // Save currently selected keywords
                    const checkboxes = document.querySelectorAll('input[name="keyword"]:checked');
                    currentSelectedKeywords = Array.from(checkboxes).map(checkbox => checkbox.value);

                    // Load checkboxes
                    const checkboxGroup = document.getElementById('checkbox-group');
                    checkboxGroup.innerHTML = '';
                    keywords.forEach(keyword => {
                        const checkboxItem = document.createElement('div');
                        checkboxItem.className = 'checkbox-item';

                        const checkbox = document.createElement('input');
                        checkbox.type = 'checkbox';
                        checkbox.name = 'keyword';
                        checkbox.value = keyword;
                        checkbox.checked = true; // Default all checkboxes to checked

                        const label = document.createElement('label');
                        label.className = 'checkbox-label';
                        label.textContent = keyword;

                        checkboxItem.appendChild(checkbox);
                        checkboxItem.appendChild(label);
                        checkboxGroup.appendChild(checkboxItem);

                        // Restore checkbox state
                        if (currentSelectedKeywords.includes(keyword)) {
                            checkbox.checked = true;
                        }

                        // Add change event listener
                        checkbox.addEventListener('change', updateTable);
                    });

                    // Update table
                    function updateTable() {
                        const tableBody = document.getElementById('paper-table-body');
                        tableBody.innerHTML = '';

                        const selectedKeywords = Array.from(document.querySelectorAll('input[name="keyword"]:checked'))
                                                      .map(checkbox => checkbox.value);

                        const filteredPapers = selectedKeywords.length > 0 ? 
                            results.filter(paper => paper.keywords.some(keyword => selectedKeywords.includes(keyword))) :
                            [];

                        filteredPapers.forEach(paper => {
                            const row = document.createElement('tr');

                            const titleCell = document.createElement('td');
                            titleCell.textContent = paper.title;
                            row.appendChild(titleCell);

                            const publishedCell = document.createElement('td');
                            publishedCell.textContent = paper.published;
                            row.appendChild(publishedCell);

                            const keywordCell = document.createElement('td');
                            keywordCell.textContent = paper.keywords;
                            row.appendChild(keywordCell);

                            const linkCell = document.createElement('td');
                            const link = document.createElement('a');
                            link.href = paper.link;
                            link.textContent = "Link";
                            link.target = "_blank";
                            link.addEventListener('click', (event) => {
                                event.preventDefault();
                                handleLinkClick(paper.link, link.href);
                            });
                            linkCell.appendChild(link);
                            row.appendChild(linkCell);

                            const localLinkCell = document.createElement('td');
                            localLinkCell.className = 'local-link-cell';
                            if (paper.local_link) {
                                const localLink = document.createElement('a');
                                localLink.href = `/local-files/${paper.local_link}`;
                                localLink.textContent = "Local Link";
                                localLink.target = "_blank";
                                localLinkCell.appendChild(localLink);
                            } else {
                                const downloadButton = document.createElement('button');
                                downloadButton.className = 'download-button';
                                downloadButton.textContent = 'Download';
                                downloadButton.addEventListener('click', () => handleDownload(paper.link, downloadButton));
                                localLinkCell.appendChild(downloadButton);
                            }
                            row.appendChild(localLinkCell);

                            tableBody.appendChild(row);
                        });
                    }

                    // Initial table load
                    updateTable();
                });
        }

        function handleLinkClick(url, href) {
            fetch('/handle-link-click', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({ url: url })
            })
            .then(response => response.json())
            .then(data => {
                console.log("Link click handled:", data);
                window.open(href, '_blank');
            });
        }

        function handleDownload(url, button) {
            button.textContent = 'Downloading...';
            button.disabled = true;
            fetch('/handle-download', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({ url: url })
            })
            .then(response => response.json())
            .then(data => {
                console.log("Download handled:", data);
                if (data.status === "success") {
                    const localLink = document.createElement('a');
                    localLink.href = `/local-files/${data.local_link}`;
                    localLink.textContent = "Local Link";
                    localLink.target = "_blank";
                    button.parentElement.appendChild(localLink);
                    button.remove();
                } else {
                    button.textContent = 'Download';
                    button.disabled = false;
                }
            });
        }

        // Initial load
        document.addEventListener('DOMContentLoaded', fetchUpdates);

        // Fetch updates when refresh button is clicked
        document.addEventListener('click', event => {
            if (event.target.matches('.refresh-button')) {
                fetchUpdates();
            }
        });
    </script>
</body>
</html>
    '''
    return render_template_string(template, updates=updates, keywords_show=keywords_show)

@app.route('/get-updates')
def get_updates_route():
    updates = get_updates()
    return jsonify(updates)

@app.route('/handle-link-click', methods=['POST'])
def handle_link_click_route():
    data = request.get_json()
    url = data.get('url')
    result = handle_link_click(url)
    return jsonify(result)

@app.route('/handle-download', methods=['POST'])
def handle_download_route():
    data = request.get_json()
    url = data.get('url')
    result = handle_download(url)
    return jsonify(result)

@app.route('/local-files/<path:filename>')
def serve_local_files(filename):
    cwd = os.getcwd()
    with open(os.path.join(cwd, "local.txt"), 'r') as file:
        local= file.read()
    local_files_directory = os.path.join(local, "papers")
    return send_from_directory(local_files_directory, filename)

def open_browser():
    webbrowser.open_new("http://127.0.0.1:5000/")



if __name__ == "__main__":
    download_mode = sys.argv[1]
    length = int(sys.argv[2])
    keywords_show = []
    for i in range(3, 3+length):
        keywords_show.append(sys.argv[i])
    print(keywords_show)
    
    print(download_mode)
    
    threading.Timer(1, open_browser).start()  # 自动打开浏览器
    with app.app_context():
        app.run(debug=True)



import requests
import os
import sqlite3
import json

def init_db():
    cwd = os.getcwd()
    with open(os.path.join(cwd, "local.txt"), 'r') as file:
        local=file.read()
    conn = sqlite3.connect(os.path.join(local, "download.db"))
    cursor = conn.cursor()
    cursor.execute('''CREATE TABLE IF NOT EXISTS papers
                     (id TEXT PRIMARY KEY, title TEXT, keyword TEXT, local_link TEXT, date TEXT)''')
    conn.commit()
    conn.close()

def get_paper_by_id(paper_id):
    cwd = os.getcwd()
    with open(os.path.join(cwd, "local.txt"), 'r') as file:
        local=file.read()
    conn = sqlite3.connect(os.path.join(local, "download.db"))
    c = conn.cursor()
    c.execute("SELECT * FROM papers WHERE id = ?", (paper_id,))
    paper = c.fetchone()
    conn.close()
    return paper

def is_paper_in_db(paper_id):
    cwd = os.getcwd()
    with open(os.path.join(cwd, "local.txt"), 'r') as file:
        local=file.read()
    conn = sqlite3.connect(os.path.join(local, "download.db"))
    cursor = conn.cursor()
    cursor.execute("SELECT 1 FROM papers WHERE id = ?", (paper_id,))
    exists = cursor.fetchone() is not None
    conn.close()
    return exists

def save_db(update):
    paper = get_paper_by_id(update['link'])
    cwd = os.getcwd()
    with open(os.path.join(cwd, "local.txt"), 'r') as file:
        local=file.read()
    conn = sqlite3.connect(os.path.join(local, "download.db"))
    cursor = conn.cursor()
    
    if paper:
        # 如果 ID 存在，检查并更新关键字字段
        existing_keywords = json.loads(paper[2])
        new_keywords = update['keyword']
        updated_keywords = list(set(existing_keywords + new_keywords))
        cursor.execute("UPDATE papers SET keyword = ? WHERE id = ?", (json.dumps(updated_keywords), update['link']))
    else:
        # 如果 ID 不存在，插入新记录
        cursor.execute("INSERT INTO papers (id, title, keyword, date, local_link) VALUES (?, ?, ?, ?, ?)",
                       (update['link'], update['title'], json.dumps(update['keyword']), str(update['published']), ""))
    
    conn.commit()
    conn.close()
    return True
def extract_arxiv_id(url):
    parts = url.split('/')
    return parts[len(parts)-1]
    
def save_paper(url, update = None, proxy = ""):
    
    cwd = os.getcwd()
    with open(os.path.join(cwd, "local.txt"), 'r') as file:
        local=file.read()
    if update == None:
        pdf_url = url.replace('abs', 'pdf')
    else:
        pdf_url = update['link'].replace('abs', 'pdf')
    paper = get_paper_by_id(pdf_url.replace("pdf", "abs"))
    conn = sqlite3.connect(os.path.join(local, "download.db"))
    cursor = conn.cursor()
    cursor.execute("SELECT 1 FROM papers WHERE id = ? and local_link = ?", (pdf_url.replace("pdf", "abs"), ""))
    exists = cursor.fetchone() is not None
    if not exists:
        return
    if proxy:
        proxies = {
            'http': f'{proxy}',
            'https': f'{proxy}',
        }
        response = requests.get(pdf_url, proxies=proxies)
    else:
        response = requests.get(pdf_url)
    
    directory = os.path.join(local,'papers')
    os.makedirs(directory, exist_ok=True)
    id = extract_arxiv_id(pdf_url)
    filename = f"{id}.pdf"
    filepath = os.path.join(directory, filename)
    with open(filepath, 'wb') as file:
        file.write(response.content)
    
    
    
    if paper:
        cursor.execute("UPDATE papers SET local_link = ? WHERE id = ?", (filename, pdf_url.replace("pdf", "abs")))
    else:
        raise Exception
    conn.commit()
    conn.close()


def down_load(updates, keywords, download_mode, proxy = ""):
    init_db()
    for keyword in keywords:
        for update in updates:
            if any(key == keyword for key in update["keyword"]):
                if save_db(update):
                    if download_mode == "2":
                        save_paper(update["link"], update, proxy)


def fetch_all_papers():
    cwd = os.getcwd()
    conn = sqlite3.connect(os.path.join(cwd, "download.db"))
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM papers")
    rows = cursor.fetchall()
    conn.close()
    
    papers = []
    for row in rows:
        paper = {
            'id': row[0],
            'title': row[1],
            'keyword': json.loads(row[2]),  # 将关键字从 JSON 字符串转换回列表
            'local_link': row[3],
            'date': row[4]
        }
        papers.append(paper)
    
    for paper in papers:
        print(paper)

def update_local_link(paper_id, local_link):
    cwd = os.getcwd()
    conn = sqlite3.connect(os.path.join(cwd, "download.db"))
    cursor = conn.cursor()
    cursor.execute("UPDATE papers SET local_link = ? WHERE id = ?", (local_link, paper_id))
    conn.commit()
    conn.close()

if __name__=="__main__":
    updates = [{'link': 'id1', 'title': 'title1', 'keyword': ['AI'], 'published': '2023-08-07 12:00:00'}, ]
    keywords = ['AI', 'Machine Learning']
    proxy = None
    
    #down_load(updates, keywords, proxy)
    fetch_all_papers()
    #updates = [{'link': 'id1', 'title': 'title1', 'keyword': ['Machine Learning'], 'published': '2023-08-07 12:00:00'}, ]
    #down_load(updates, keywords, proxy)
    #fetch_all_papers()




from email.header import Header
from email.mime.text import MIMEText
from email.utils import parseaddr, formataddr
import smtplib
from email.mime.multipart import MIMEMultipart

def _format_addr(s):
    name, addr = parseaddr(s)
    return formataddr((Header(name, 'utf-8').encode(), addr))

def send_email(updates, from_addr, password, to_addr, smtp_server, smtp_port):
    subject = "Newest & Latest arXiv Updates"
    body = "\n".join([f"Title: {update['title']}\nKeyword: {update['keyword']}\nPublished: {update['published']}\nLink: {update['link']}\n" for update in updates])

    msg = MIMEMultipart()
    msg['From'] = _format_addr('ArXivFans <%s>' % from_addr)
    msg['To'] = _format_addr('Dear user <%s>' % to_addr)
    msg['Subject'] = subject

    msg.attach(MIMEText(body, 'plain'))

    try:
        server = smtplib.SMTP(smtp_server, smtp_port)
        #server.starttls()
        server.login(from_addr, password)

        server.sendmail(from_addr, [to_addr], msg.as_string())
        print("message sent successfully")
        server.quit()
    except smtplib.SMTPAuthenticationError as e:
        print("incorrect password")
        print("fail to send the email")
    except Exception as e:
        print(f"error: {e}")



from setuptools import setup, find_packages

setup(
    name='arXivFans',  
    version='0.3.1',
    description='This project provides an effective way to fetch the latest papers from arXiv and view them through email notifications or a web interface.',
    url='https://github.com/daihangpku/arXivFans.git',  
    author='Daihang',
    author_email='daihang2300012956@163.com',
    license='MIT',  
    packages=find_packages(),
    install_requires=[
        'arxiv',
        'flask',
        'requests',
        'schedule',
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',  
    entry_points = {
    'console_scripts': ['fetch = fetcharxiv.main:main', 'web = fetcharxiv.webpage:main']
    }

)
